# lecture code for KRLS modeling in R
# by J. Esarey, 2/20/2017 
#
# examples and (most of the) code taken from the Hainmuller and Hazlett (2014)
#        replication file: http://hdl.handle.net/1902.1/22095


library(KRLS)

### Additional Figures in Appendix
##################################################
## Figure A.1  Fitting a Simple Function with KRLS
X=1:4
y=-2:1
N=4
x_index=seq(-1,6,.01)
s2=3

# kernel matrix
K <- gausskernel(X,sigma=s2)
# get c's
out <- krls(X=X,y=y,sigma=s2,vcov=FALSE,deriv=FALSE,print.level=0)
# get Yhats for x_index
pout <- predict(out,newdata=x_index)

# get gaussians for plot
store <- storescaled <- list()
for(j in 1:N){
  store[[j]]       <- exp(-((x_index-X[j])^2/s2)) # unscaled
  storescaled[[j]] <- out$c[j]*exp(-((x_index-X[j])^2/s2)) # scaled
}
store       <- matrix(unlist(store),ncol=length(x_index),byrow=T)
storescaled <- matrix(unlist(storescaled),ncol=length(x_index),byrow=T)

# plot unscaled Gaussians
###pdf("figureA1a.pdf",width=7,height=6)
matplot(t(store),x=x_index,type="l",
        col="black",lty="dotted",
        xlim=c(.5,4.5),ylim=c(-3,3),
        lwd=3,
        xlab="x",ylab="y")
points(y=rep(0,N),x=X,col="black",pch=19,cex=1.2)
legend("bottomright",
       legend=c("Unscaled Gaussians"),
       lty=c("dotted"),
       lwd=c(3),cex=1,bg="white")
###dev.off()

# plot scaled Gaussians
###pdf("figureA1b.pdf",width=7,height=6)
matplot(t(storescaled),x=x_index,type="l",
        col="black",lty="dotted",
        xlim=c(.5,4.5),ylim=c(-3,3),
        lwd=3,
        xlab="x",ylab="y")
lines(pout$fit,x=x_index,lwd=3)
points(y,x=X,pch=19,col="black",cex=2)
legend("bottomright",
       legend=c("Scaled Gaussians","KRLS Fit (Superposition)"),
       lty=c("dotted","solid"),
       lwd=c(3,3),cex=1,bg="white")
###dev.off()






#########################################################
## Table 4: Predictors of Genocide Onset: OLS versus KRLS
## Harff 2003 AJPS

rm(list=ls())
library(pROC)
library(lattice)
library(KRLS)
library(boot)

load("Harff2003.RData")
colnames(d)=c("Genonset","PriorUpheaval","PriorGen","IdeologicalChar",
              "AUTOC","EthnicChar","TradeOpen")

out <- krls(y=d[,1],X=d[,-1],derivative = TRUE,binary=TRUE,print.level=0)
summary(out)

olsout <- lm(as.formula(paste(names(d)[1],"~",paste(names(d)[-1],collapse="+"),sep="")),data=d)
summary(olsout)

# create table
olstab <-  c(  summary(olsout)$coefficients[,1],
               summary(olsout)$coefficients[,2])[c(1,8,2,9,3,10,4,11,5,12,6,13,7,14)]
krlstab<- c(NA,NA,c(out$avgderivatives[1,],
                    sqrt(out$var.avgderivatives[1,]))[c(1,7,2,8,3,9,4,10,5,11,6,12)])

krlsq  <- rbind(summary(out)$qcoefficients,rep(NA,3))[c(7,7,1,7,2,7,3,7,4,7,5,7,6,7),]
table4 <- round(cbind(cbind(olstab,krlstab),krlsq),3)

table4

###pdf("Figure5.pdf",width=10,height=8)
plot(out,which=1,
     scales=list(alternating = FALSE,
                 relation="same",yaxt="i"),
     xlab="marginal effect",col=grey(.4),
     par.settings  = list(
       strip.background = list(col = grey(.95) )),
     layout=c(3,2),type="percent")
###dev.off()

# effect of prior upheaval score on genocide onset
plot(d[,"PriorUpheaval"],y=out$derivatives[,"PriorUpheaval"],col=16,pch=16,
     xlab="Prior Upheaval Score",ylab="Marginal Effect of Prior Upheaval")
lines(lwd=2,lowess(f=1,x=d[,"PriorUpheaval"],y=out$derivatives[,"PriorUpheaval"]))
abline(h=coefficients(olsout)[2], lty=2)


# bootstrap standard errors
boot.krls <- function(dat.in, idx){
  
  d.boot <- dat.in[idx,]
  krlsout.boot=krls(y=d.boot[,1],X=d.boot[,-1],derivative = TRUE,binary=TRUE,print.level=0)
  lo.mod <- loess(krlsout.boot$derivatives[,"PriorUpheaval"] ~ d.boot[,"PriorUpheaval"], span=1)
  out <- suppressWarnings(predict(lo.mod, newdata=seq(from=min(dat.in$PriorUpheaval), to=max(dat.in$PriorUpheaval), by=1)))
  return(out)
  
}
boot.store <- boot(data=d, statistic=boot.krls, R=1000)$t
upheaval.plot <- seq(from=min(d$PriorUpheaval), to=max(d$PriorUpheaval), by=1)
ci.hi <- apply(X=boot.store, MARGIN=2, FUN=quantile, probs=0.975, na.rm=TRUE)
ci.lo <- apply(X=boot.store, MARGIN=2, FUN=quantile, probs=0.025, na.rm=TRUE)
plot(d[,"PriorUpheaval"],y=out$derivatives[,"PriorUpheaval"],col=16,pch=16,
     xlab="Prior Upheaval Score",ylab="Marginal Effect of Prior Upheaval",
     ylim=c(-0.070, 0.040))
lines(lwd=2,lowess(f=1,x=d[,"PriorUpheaval"],y=out$derivatives[,"PriorUpheaval"]))
abline(h=coefficients(olsout)[2], lty=2)
lines(lwd=2, lty=2, ci.lo ~ upheaval.plot )
lines(lwd=2, lty=2, ci.hi ~ upheaval.plot )



# effect of prior genocide on genocide onset
d$PriorGen.lab <- factor(d$PriorGen, levels=c(0,1), labels=c("No", "Yes"))
boxplot(out$derivatives[,"PriorGen"]~d[,"PriorGen.lab"],
     xlab="Prior Genocide",ylab="Marginal Effect of Prior Genocides")
abline(h=coefficients(olsout)[3], lty=2)


# roc test vs logit
rockrls <- roc(d$Genonset,out$fit)
logit <- glm(as.formula(paste(names(d)[1],"~",paste(names(d)[-1],collapse="+"),sep="")),
             data=d,family=binomial(link = "logit"))
roclogit  <- roc(d$Genonset,logit$fitted)
roc.test(rockrls,roclogit)













## Additional Empirical Example based on Brambor et al. 2006.
## Data from Golder M. Golder, 2006 (AJPS)
rm(list=ls())

library(KRLS)
library(dotwhisker)
library(mvtnorm)
library(boot)


## A.7.   Plot of pointwise partial derivatives
#Load prepared data from Golder (includes pre-processing done according to posted replication file)
load("GolderPrepared.Rdata")
d=Golder
rm(Golder)
covars=c("proximity1", "enpres", "eneg", "logmag")
#Run KRLS model.  To replicated original, use only data where old==1

d_clean=na.omit(d[,c("enep1",covars,"old","country")])
d_clean=d_clean[d_clean$old==1,]
#To replicated original, use only data where old==1
krlsout=krls(y=d_clean[,"enep1"],X=d_clean[,covars],print.level=0)

summary(krlsout)

avg.der <- krlsout$avgderivatives
var.der <- krlsout$var.avgderivatives
coef.frame <- data.frame(term=colnames(avg.der), estimate=as.vector(avg.der), std.error=as.vector(sqrt(var.der)))

dwplot(coef.frame) + relabel_y_axis(c("Proximity of Presidential Election", "# of Presidential Candidates", "Ethnolinguistic Fractionalization", "log Avg. District Magnitude")) + xlab("Coefficient Estimate") + ylab("")

#pdf("figureA7.pdf",height=5,width=6)
plot(d_clean[,"enpres"],y=krlsout$derivatives[,"proximity1"],col=16,pch=16,
     xlab="Effective Presidential Candidates",ylab="Marginal Effect of Temporal Proximity")
lines(lwd=2,lowess(f=1,x=d_clean[,"enpres"],y=krlsout$derivatives[,"proximity1"]))
#legend("bottomright",legend=c("pointwise","lowess fit"),lty=c(0,1),pch=c(20,NA),col=c(16,1))
#dev.off()

# bootstrap standard errors
boot.krls <- function(dat.in, idx){
  
  d_clean.boot <- dat.in[idx,]
  krlsout.boot=krls(y=d_clean.boot[,"enep1"],X=d_clean.boot[,covars],print.level=0)
  lo.mod <- loess(krlsout.boot$derivatives[,"proximity1"] ~ d_clean.boot[,"enpres"], span=1)
  out <- suppressWarnings(predict(lo.mod, newdata=seq(from=min(d_clean$enpres), to=max(d_clean$enpres), by=0.1)))
  return(out)
  
}
boot.store <- boot(data=d_clean, R=100, statistic=boot.krls)$t
pres.plot <- seq(from=min(d_clean$enpres), to=max(d_clean$enpres), by=0.1)
ci.hi <- apply(X=boot.store, MARGIN=2, FUN=quantile, probs=0.975, na.rm=TRUE)
ci.lo <- apply(X=boot.store, MARGIN=2, FUN=quantile, probs=0.025, na.rm=TRUE)
plot(d_clean[,"enpres"],y=krlsout$derivatives[,"proximity1"],col=16,pch=16,
     xlab="Effective Presidential Candidates",ylab="Marginal Effect of Temporal Proximity")
lines(lwd=2,lowess(f=1,x=d_clean[,"enpres"],y=krlsout$derivatives[,"proximity1"]))
lines(lwd=2,lowess(f=1,x=d_clean[,"enpres"],y=krlsout$derivatives[,"proximity1"]))
lines(lwd=2, lty=2, ci.lo ~ pres.plot )
lines(lwd=2, lty=2, ci.hi ~ pres.plot )